import struct

SENSOR_MAP = {
    "ax": 0,
    "ay": 1,
    "az": 2,
    "gx": 3,
    "gy": 4,
    "gz": 5,
    "roll": 6,
    "pitch": 7,
    "yaw": 8,
    "lat": 9,
    "lng": 10,
}


SENSOR_FOMAT = {
    0x01: {
        'type':'imu',
        'format': 'float',
        'length': 6,
    },
    0x02: {
        'type':'mag',
        'format': 'float',
        'length': 3,
    },
    0x03: {
        'type':'gps',
        'format': 'float',
        'length': 12,
    },
    0x04: {
        'type':'tof1',
        'format': 'int16',
        'length': 16,
    },
    0x05: {
        'type':'tof2',
        'format': 'int16',
        'length': 16,
    },
}
         
def _unpack(data, format):
    return struct.unpack(format, data)[0]

def _checksum(data):
    '''
    calculate the checksum of the data
    checksum = xor(payload)
    '''    
    checksum = 0x00  # Initialize the checksum as an integer
    for d in data:
        checksum ^= d
    return checksum

def g_unpack(data, sensor_info, checksum, length):
    '''
    general unpack function
    '''
    format = sensor_info['format']
    if format =='float':
        format = 'f'
        size  = 4
    elif format == 'int16':
        format = 'h'
        size = 2        
    elif format == 'int32':
        format = 'i'
        size = 4
    elif format == 'uint8':
        format = 'B'
        size = 1
    elif format == 'uint16':
        format = 'H'
        size = 2
    elif format == 'uint32':
        format = 'I'
        size = 4
    else:
        raise ValueError(f'Format {format} is not supported')
    payload_size = sensor_info['length']*size
    if length != payload_size:
        raise ValueError(f'Data length is not {payload_size}')
    if _checksum(data) != checksum:
        raise ValueError('Checksum does not match')
    r = [_unpack(data[i:i+size], format) for i in range(0, length, size)]
    return r
